@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008, 2010, 2012 Sergey Poznyakoff
+   Copyright (C) 2008-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ lev_sel(int cmd, dico_key_t key, const char *dict_word)
 {
     if (cmd == DICO_SELECT_RUN) {
 	int dist = dico_levenshtein_distance(key->word, dict_word,
-					     (int)key->strat->closure);
+					     *(int*)key->strat->closure);
 	if (dist < 0)
 	    return 0;
 	return dist <= levenshtein_distance;
@@ -31,29 +31,34 @@ lev_sel(int cmd, dico_key_t key, const char *dict_word)
     return 0;
 }
 
+static int lev_flags = 0;
+static int nlev_flags = DICO_LEV_NORM;
+static int dlev_flags = DICO_LEV_DAMERAU;
+static int ndlev_flags = DICO_LEV_NORM | DICO_LEV_DAMERAU;
+
 static struct dico_strategy levstrat[] = {
     { "lev",
       "Match headwords within given Levenshtein distance",
       lev_sel,
-      NULL },
+      &lev_flags },
     { "nlev",
       "Match headwords within given Levenshtein distance (normalized)",
       lev_sel,
-      (void*)DICO_LEV_NORM },
+      &nlev_flags },
     { "dlev",
       "Match headwords within given Damerau-Levenshtein distance",
       lev_sel,
-      (void*)DICO_LEV_DAMERAU },
+      &dlev_flags },
     { "ndlev",
       "Match headwords within given Damerau-Levenshtein distance (normalized)",
       lev_sel,
-      (void*)(DICO_LEV_NORM|DICO_LEV_DAMERAU) }
+      &ndlev_flags }
 };
 
 static void
 dicod_xlevdist(dico_stream_t str, int argc, char **argv)
 {
-    if (c_strcasecmp(argv[1], "tell") == 0) 
+    if (c_strcasecmp(argv[1], "tell") == 0)
 	stream_printf(str, "280 %d\n", levenshtein_distance);
     else if (isdigit(argv[1][0]) && argv[1][0] != '0' && argv[1][1] == 0) {
 	levenshtein_distance = atoi(argv[1]);
@@ -62,9 +67,9 @@ dicod_xlevdist(dico_stream_t str, int argc, char **argv)
     } else
 	stream_writez(str, "500 invalid argument\n");
 }
-	
+
 void
-register_lev()
+register_lev(void)
 {
     int i;
     static struct dicod_command cmd[] = {
@@ -77,5 +82,3 @@ register_lev()
     dico_set_default_strategy("nlev");
     dicod_capa_register("xlev", cmd, NULL, NULL);
 }
-
-	  
