@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008, 2010, 2012 Sergey Poznyakoff
+   Copyright (C) 2008-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ dicod_markup(dico_stream_t str, int argc, char **argv)
 }
 
 void
-register_markup()
+register_markup(void)
 {
     static struct dicod_command cmd[] = {
 	{ "OPTION MARKUP", 2, 3, "type", "Set output markup type",
@@ -47,18 +47,20 @@ register_markup()
 #define MARKUP_CAPA_PREFIX "markup-"
 
 void
-markup_flush_capa()
+markup_flush_capa(void)
 {
-    dico_iterator_t itr;
-    const char *p;
-    
-    itr = xdico_list_iterator(dico_markup_list);
-    for (p = dico_iterator_first(itr); p; p = dico_iterator_next(itr)) {
-	size_t len = sizeof(MARKUP_CAPA_PREFIX) + strlen(p);
-	char *str = xmalloc(len);
-	strcat(strcpy(str, MARKUP_CAPA_PREFIX), p);
-	dicod_capa_register(str, NULL, NULL, NULL);
-	dicod_capa_add(str);
+    if (dicod_capa_is_enabled("markup")) {
+	dico_iterator_t itr;
+	const char *p;
+
+	itr = xdico_list_iterator(dico_markup_list);
+	for (p = dico_iterator_first(itr); p; p = dico_iterator_next(itr)) {
+	    size_t len = sizeof(MARKUP_CAPA_PREFIX) + strlen(p);
+	    char *str = xmalloc(len);
+	    strcat(strcpy(str, MARKUP_CAPA_PREFIX), p);
+	    dicod_capa_register(str, NULL, NULL, NULL);
+	    dicod_capa_add(str);
+	}
+	dico_iterator_destroy(&itr);
     }
-    dico_iterator_destroy(&itr);
 }
