@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2012-2016 Sergey Poznyakoff
+   Copyright (C) 2012-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,14 +14,12 @@
    You should have received a copy of the GNU General Public License
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <dico.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <stdio.h>
-#include <stdlib.h>    
+#include <stdlib.h>
 #include <errno.h>
 #include <sysexits.h>
 #include <string.h>
@@ -31,7 +29,7 @@ static void
 usage(FILE *fp)
 {
     fprintf(fp, "usage: %s [-debug] [-struct] [-nopr] [-help] FILE [OFF SIZE]\n", dico_program_name);
-}    
+}
 
 struct output_closure {
     FILE *stream;
@@ -49,39 +47,42 @@ print_tag(int end, struct gcide_tag *tag, void *data)
 	clos->level--;
 	printf("%*.*s", 2*clos->level,2*clos->level, "");
 	printf("END");
-	if (tag->tag_parmc)
+	if (tag->v.tag.tag_parmc)
 	    printf(" %s", tag->tag_name);
 	putchar('\n');
 	return 0;
     }
     printf("%*.*s", 2*clos->level,2*clos->level, "");
     switch (tag->tag_type) {
-    case gcide_content_unspecified:
+    case gcide_content_top:
+	printf("BEGIN\n");
+	clos->level++;
 	break;
     case gcide_content_text:
 	printf("TEXT");
-	for (i = 0; i < tag->tag_parmc; i++)
-	    printf(" %s", tag->tag_parmv[i]);
-	printf(":\n%s\n%*.*sENDTEXT", tag->tag_v.text,
+	printf(":\n%s\n%*.*sENDTEXT", tag->v.text,
 	       2*clos->level, 2*clos->level, "");
-	if (tag->tag_parmc)
-	    printf(" %s", tag->tag_name);
 	putchar('\n');
 	break;
-    case gcide_content_taglist:
+    case gcide_content_tag:
 	printf("BEGIN");
-	for (i = 0; i < tag->tag_parmc; i++)
-	    printf(" %s", tag->tag_parmv[i]);
+	for (i = 0; i < tag->v.tag.tag_parmc; i++)
+	    printf(" %s", tag->v.tag.tag_parmv[i]);
 	putchar('\n');
 	clos->level++;
 	break;
+    case gcide_content_nl:
+	printf("NL\n");
+	break;
+    case gcide_content_br:
+	printf("BR\n");
     }
     return 0;
 }
 
 #define GCIDE_NOPR 0x0000001
-#define GOF_IGNORE 0x0001000
-#define GOF_AS     0x0002000
+#define GOF_INIT   0x0001000
+#define GOF_IGNORE 0x0002000
 
 static int
 print_text(int end, struct gcide_tag *tag, void *data)
@@ -91,28 +92,15 @@ print_text(int end, struct gcide_tag *tag, void *data)
     static char *ref[2] = { "{" , "}" };
 
     switch (tag->tag_type) {
-    case gcide_content_unspecified:
+    case gcide_content_top:
 	break;
     case gcide_content_text:
 	if (clos->flags & GOF_IGNORE)
 	    break;
-	if (clos->flags & GOF_AS) {
-	    char *s = tag->tag_v.text;
-	    
-	    if (strncmp(s, "as", 2) == 0 &&
-		(isspace(s[3]) || ispunct(s[3]))) {
-		fwrite(s, 3, 1, clos->stream);
-		for (s += 3; *s && isspace(*s); s++)
-		    fputc(*s, clos->stream);
-		fprintf(clos->stream, "%s%s", quote[0], s);
-	    } else
-		fprintf(clos->stream, "%s", quote[0]);
-	} else
-	    fprintf(clos->stream, "%s", tag->tag_v.text);
+	fprintf(clos->stream, "%s", tag->v.text);
 	break;
-    case gcide_content_taglist:
-	if (tag->tag_parmc) {
-	    clos->flags &= ~GOF_AS;
+    case gcide_content_tag:
+	if (tag->v.tag.tag_parmc) {
 	    if (end) {
 		if (strcmp(tag->tag_name, "pr") == 0 &&
 			 clos->flags & GCIDE_NOPR)
@@ -123,20 +111,37 @@ print_text(int end, struct gcide_tag *tag, void *data)
 		    fprintf(clos->stream, "%s", quote[1]);
 		else if (strcmp(tag->tag_name, "er") == 0)
 		    fprintf(clos->stream, "%s", ref[1]);
+		else if (strcmp(tag->tag_name, "source") == 0)
+		    fputc(']', clos->stream);
 	    } else {
 		if (strcmp(tag->tag_name, "pr") == 0 &&
 			 clos->flags & GCIDE_NOPR)
 		    clos->flags |= GOF_IGNORE;
 		else if (clos->flags & GOF_IGNORE)
 		    break;
-		else if (strcmp(tag->tag_name, "sn") == 0)
-		    fputc('\n', clos->stream);
-		else if (strcmp(tag->tag_name, "as") == 0)
-		    clos->flags |= GOF_AS;
+
+		if (gcide_is_block_tag(tag)) {
+		    if (clos->flags & GOF_INIT)
+			fputc('\n', clos->stream);
+		    else
+			clos->flags |= GOF_INIT;
+		}
+
+		if (strcmp(tag->tag_name, "as") == 0)
+		    fprintf(clos->stream, "%s", quote[0]);
 		else if (strcmp(tag->tag_name, "er") == 0)
 		    fprintf(clos->stream, "%s", ref[0]);
+		else if (strcmp(tag->tag_name, "source") == 0)
+		    fputc('[', clos->stream);
 	    }
 	}
+	break;
+    case gcide_content_nl:
+	fputc(' ', clos->stream);
+	break;
+    case gcide_content_br:
+	fputc('\n', clos->stream);
+	break;
     }
     return 0;
 }
@@ -152,8 +157,10 @@ main(int argc, char **argv)
     struct gcide_parse_tree *tree;
     struct output_closure clos;
     int show_struct = 0;
+    int check_only = 0;
     int dbglex = 0;
-    
+    struct gcide_locus locus;
+
     dico_set_program_name(argv[0]);
     clos.flags = 0;
     clos.stream = stdout;
@@ -170,6 +177,8 @@ main(int argc, char **argv)
 	    show_struct = 1;
 	else if (strcmp(arg, "-nopr") == 0)
 	    clos.flags = GCIDE_NOPR;
+	else if (strcmp(arg, "-c") == 0 || strcmp(arg, "--check") == 0)
+	    check_only = 1;
 	else if (strcmp(arg, "--") == 0) {
 	    --argc;
 	    ++argv;
@@ -202,16 +211,16 @@ main(int argc, char **argv)
 	if (st.st_size < offset) {
 	    dico_log(L_ERR, 0, "invalid offset");
 	    exit(EX_UNAVAILABLE);
-	}   
+	}
 	size = st.st_size - offset;
     }
-    
+
     textbuf = malloc(size);
     if (!textbuf) {
-	dico_log(L_ERR, 0, "not enough memory");
+	DICO_LOG_ERRNO();
 	exit(EX_UNAVAILABLE);
     }
-    
+
     fp = fopen(file, "r");
     if (!fp) {
 	dico_log(L_ERR, errno, "cannot open file %s", file);
@@ -227,14 +236,21 @@ main(int argc, char **argv)
 	dico_log(L_ERR, errno, "%s: read error", file);
 	exit(EX_UNAVAILABLE);
     }
-	
-    tree = gcide_markup_parse(textbuf, size, dbglex);
+
+    locus.file = file;
+    locus.offset = offset;
+    tree = gcide_markup_parse(textbuf, size, dbglex, &locus);
     if (!tree)
 	exit(EX_UNAVAILABLE);
 
-    clos.level = 0;
-    gcide_parse_tree_inorder(tree, show_struct ? print_tag : print_text, &clos);
-    
+    if (!check_only) {
+	clos.level = 0;
+	gcide_parse_tree_inorder(tree, show_struct ? print_tag : print_text,
+				 &clos);
+	if (!show_struct)
+	    fputc('\n', stdout);
+    }
+
     gcide_parse_tree_free(tree);
     exit(0);
 }
